# coding: utf-8

"""
    Lumos Connectors

    # The Lumos Connector API ## Intro This API is a self-describing API which allows third-party systems to easily integrate with Lumos. It provides endpoints to retrieve information about each connector, their capabilities.  We call individual libraries that proxy Lumos requests to app-specific requests, and then app-specific responses to Lumos responses, a **connector**.  ## How to use this API 1. You can call the `list_connector_app_ids` to see the list of available connector app IDs, e.g. `pagerduty`. One app ID is required to call most other APIs. 2. Calling `info` explains * what kind of authentication credentials the integration requires * which capabilities the integration supports * what input parameters are required for each capability for the given connector  ## Authenticating with a connector There are two broad categories of authentication - Shared Secret (e.g. API keys) and OAuth-like.  TODO: explain more details - look at `info.authentication_schema` - if it's OAuth - setup needs to call - `get_authorization_url` - `handle_authorization_callback` - steady state needs to call `refresh_access_token` - `validate_credentials` will check that your creds are good, and let you know what the unique tenant ID is  ## Reading customer account data TODO: write more docs - `list_resources` - `list_entitlements` - `find_entitlement_associations`  ## Pagination TODO: write more docs - `response.page.token` may contain an opaque string - pass this string back to the next request `request.page.token` to get the next set of data

    The version of the OpenAPI document: 0.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from connector.generated.models.client_authentication_method import ClientAuthenticationMethod
from connector.generated.models.o_auth_capabilities import OAuthCapabilities
from connector.generated.models.o_auth_flow_type import OAuthFlowType
from connector.generated.models.o_auth_request import OAuthRequest
from connector.generated.models.o_auth_scopes import OAuthScopes
from typing import Optional, Set
from typing_extensions import Self

class OAuthSettings(BaseModel):
    """
    OAuth settings for connector Integration class, also controls the way the SDK OAuth module will be initialized.
    """ # noqa: E501
    authorization_url: StrictStr = Field(description="OAuth authorization URL for this connector.")
    token_url: StrictStr = Field(description="OAuth token URL for this connector.")
    scopes: OAuthScopes = Field(description="OAuth scopes for this connector per capability, eg. dict[CapabilityName, string].")
    flow_type: Optional[OAuthFlowType] = Field(default=None, description="SDK OAuth module flow type, defaults to CODE_FLOW if not specified.")
    client_auth: Optional[ClientAuthenticationMethod] = Field(default=None, description="SDK OAuth module client authentication method, defaults to CLIENT_SECRET_POST if not specified.")
    request_type: Optional[OAuthRequest] = Field(default=None, description="SDK OAuth module request type settings, defaults to POST + BODY if not specified.")
    capabilities: Optional[OAuthCapabilities] = Field(default=None, description="SDK OAuth module capabilities, defaults to true (enabled) for all capabilities, set to false to not register a particular capability, you can skip it or implement it manually.")
    __properties: ClassVar[List[str]] = ["authorization_url", "token_url", "scopes", "flow_type", "client_auth", "request_type", "capabilities"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of OAuthSettings from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of scopes
        if self.scopes:
            _dict['scopes'] = self.scopes.to_dict()
        # override the default output from pydantic by calling `to_dict()` of request_type
        if self.request_type:
            _dict['request_type'] = self.request_type.to_dict()
        # override the default output from pydantic by calling `to_dict()` of capabilities
        if self.capabilities:
            _dict['capabilities'] = self.capabilities.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of OAuthSettings from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "authorization_url": obj.get("authorization_url"),
            "token_url": obj.get("token_url"),
            "scopes": OAuthScopes.from_dict(obj["scopes"]) if obj.get("scopes") is not None else None,
            "flow_type": obj.get("flow_type"),
            "client_auth": obj.get("client_auth"),
            "request_type": OAuthRequest.from_dict(obj["request_type"]) if obj.get("request_type") is not None else None,
            "capabilities": OAuthCapabilities.from_dict(obj["capabilities"]) if obj.get("capabilities") is not None else None
        })
        return _obj


